/* BDF - Copyright (c) 2010-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfExtensionInitializer;
import fr.exemole.bdfserver.api.storage.BdfExtensionStorage;
import fr.exemole.bdfserver.api.storage.BdfStorageException;
import fr.exemole.bdfserver.api.storage.ResourceStorage;
import fr.exemole.bdfserver.api.storage.ResourceStorages;
import fr.exemole.bdfserver.conf.BdfServerDirs;
import fr.exemole.bdfserver.conf.Conf;
import fr.exemole.bdfserver.conf.ConfConstants;
import fr.exemole.bdfserver.conf.ConfUtils;
import fr.exemole.bdfserver.conf.dom.ConfDOMReader;
import fr.exemole.bdfserver.email.SmtpManager;
import fr.exemole.bdfserver.storage.directory.StorageFactory;
import fr.exemole.bdfserver.storage.directory.bdfdata.BdfStorageUtils;
import fr.exemole.bdfserver.storage.directory.bdfdata.StorageDirectories;
import fr.exemole.bdfserver.tools.storage.DirectoryResourceStorage;
import fr.exemole.bdfserver.tools.storage.DistResourceStorage;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import fr.exemole.bdfserver.tools.storage.Storages;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.jar.JarFile;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.buildinfo.BuildInfo;
import net.mapeadores.util.buildinfo.BuildInfoParser;
import net.mapeadores.util.exceptions.InitException;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.logging.MessageLogBuilder;
import net.mapeadores.util.servlets.ServletUtils;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public final class BDFInit {

    private BDFInit() {

    }

    public static SmtpManager smtpManager(ServletContext servletContext) {
        File smtpFile = null;
        String smtpFileParam = ServletUtils.getInitParameter(servletContext, "smtpFile");
        if (smtpFileParam != null) {
            smtpFile = new File(smtpFileParam);
        } else {
            File oldFile = ServletUtils.getRealFile(servletContext, "/WEB-INF/smtp.ini");
            if (oldFile == null) {
                oldFile = ServletUtils.getRealFile(servletContext, "/WEB-INF/bdf-smtp.properties");
            }
            if (oldFile != null) {
                smtpFile = oldFile;
            }
        }
        return new SmtpManager(smtpFile);
    }

    public static boolean loginPasswordDisabled(ServletContext servletContext) {
        return ServletUtils.getBooleanInitParameter(servletContext, "disableLoginPassword");
    }

    public static boolean authentificationSharingAllowed(ServletContext servletContext) {
        return ServletUtils.getBooleanInitParameter(servletContext, "allowAuthentificationSharing");
    }

    public static Conf conf(ServletContext servletContext) {
        String bdfConfFileParam = ServletUtils.getInitParameter(servletContext, "bdfConfFile");
        File confFile = null;
        if (bdfConfFileParam != null) {
            confFile = new File(bdfConfFileParam);
            if (!confFile.exists()) {
                confFile = null;
            }
        }
        if (confFile == null) {
            confFile = ServletUtils.getRealFile(servletContext, "/WEB-INF/bdf-conf.xml");
        }
        if (confFile == null) {
            StringBuilder buf = new StringBuilder();
            buf.append("bdfConfFile does not exist : ");
            if (bdfConfFileParam != null) {
                buf.append(bdfConfFileParam);
                buf.append(" | ");
            }
            buf.append("/WEB-INF/bdf-conf.xml");
            throw new InitException(buf.toString());
        }
        if (confFile.isDirectory()) {
            throw new InitException("bdfConfFile is a directory : " + confFile.getAbsolutePath());
        }
        String servletContextName = ServletUtils.getServletContextName(servletContext);
        File relativeDirectory = ServletUtils.getRealFile(servletContext, "/");
        if (relativeDirectory == null) {
            throw new InitException("Unable to find webapps root : /");
        }
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(confFile);
            ConfDOMReader reader = new ConfDOMReader(servletContextName, relativeDirectory, ServletUtils.getInitParameter(servletContext, "multiBdf"));
            Conf conf = reader.readConf(doc.getDocumentElement());
            return conf;
        } catch (ParserConfigurationException pce) {
            throw new InitException("ParserConfigurationException : " + pce.getMessage());
        } catch (IOException ioe) {
            throw new InitException("IOException when reading bdfConfFile : " + ioe.getMessage() + " / " + confFile.getAbsolutePath());
        } catch (SAXException sax) {
            throw new InitException("XML Error when reading bdfConfFile : " + sax.getMessage() + " / " + confFile.getAbsolutePath());
        }
    }

    public static Storages storages(BdfServerDirs dirs, ResourceStorages webappsResourceStorages) {
        checkBalayage(dirs);
        BdfExtensionStorage bdfExtensionStorage = StorageFactory.newBdfExtensionStorage(dirs);
        Storages storages = new Storages(dirs);
        storages
                .setBdfExtensionStorage(bdfExtensionStorage);
        MessageLogBuilder logBuilder = new MessageLogBuilder();
        StorageFactory.populateWithBdfdata(storages, dirs, logBuilder);
        storages
                .setInitMessageLog(logBuilder.toMessageLog())
                .setResourceStorages(initBdfServerResourceStorages(dirs, webappsResourceStorages))
                .setBalayageStorage(StorageFactory.newBalayageStorage(dirs))
                .setTableExportStorage(StorageFactory.newTableExportStorage(dirs))
                .setTemplateStorage(StorageFactory.newTemplateStorage(dirs))
                .setOutputStorage(StorageFactory.newOutputStorage(dirs))
                .setCacheStorage(StorageFactory.newCacheStorage(dirs));
        return storages;
    }

    public static BuildInfo buildInfo() {
        if (BDFInit.class.getResource("VERSION") != null) {
            try (InputStream is = BDFInit.class.getResourceAsStream("VERSION")) {
                BuildInfo buildInfo = BuildInfoParser.parse(is);
                return buildInfo;
            } catch (IOException ioe) {
                throw new InternalResourceException(ioe);
            }
        } else {
            return null;
        }
    }

    public static StorageDirectories multiStorageDirectories(Conf conf) {
        File dataDir = ConfUtils.getMultiDataDir(conf);
        File backupDir = ConfUtils.getMultiBackupDir(conf);
        return BdfStorageUtils.toStorageDirectories(dataDir, backupDir);
    }

    public static List<BdfExtensionInitializer> extensionInitializerList(Conf conf) {
        File dir = conf.getDir(ConfConstants.LIB_EXTENSIONS);
        if ((dir == null) || (!dir.exists())) {
            return Collections.emptyList();
        }
        List<BdfExtensionInitializer> bdfExtensionInitializerList;
        try {
            bdfExtensionInitializerList = extensionInitializerList(dir);
        } catch (IOException | ClassNotFoundException | InstantiationException | IllegalAccessException e) {
            throw new InitException(e);
        }
        return bdfExtensionInitializerList;
    }

    private static List<BdfExtensionInitializer> extensionInitializerList(File dir) throws IOException, ClassNotFoundException, InstantiationException, IllegalAccessException {
        Map<String, String> existingRegistrationNameMap = new HashMap<String, String>();
        String intializerClassName = BdfExtensionInitializer.class.getName();
        File[] files = dir.listFiles();
        int length = files.length;
        List<URL> urlList = new ArrayList<URL>();
        List<File> fileList = new ArrayList<File>();
        for (int i = 0; i < length; i++) {
            File file = files[i];
            if (file.isDirectory()) {
                continue;
            }
            if (!file.getName().endsWith(".jar")) {
                continue;
            }
            urlList.add(file.toURI().toURL());
            fileList.add(file);
        }
        length = urlList.size();
        if (length == 0) {
            return Collections.emptyList();
        }
        List<BdfExtensionInitializer> bdfExtensionInitializerList = new ArrayList<BdfExtensionInitializer>();
        URLClassLoader loader = new URLClassLoader(urlList.toArray(new URL[length]), BDFInit.class.getClassLoader());
        Enumeration enumeration;
        String tmp;
        Class tmpClass;
        for (int i = 0; i < length; i++) {
            File file = fileList.get(i);
            JarFile jar = new JarFile(file.getAbsolutePath());
            enumeration = jar.entries();
            while (enumeration.hasMoreElements()) {
                tmp = enumeration.nextElement().toString();
                if (tmp.length() > 6 && tmp.substring(tmp.length() - 6).compareTo(".class") == 0) {
                    tmp = tmp.substring(0, tmp.length() - 6);
                    tmp = tmp.replaceAll("/", ".");
                    tmpClass = Class.forName(tmp, true, loader);
                    Class[] interfaces = tmpClass.getInterfaces();
                    int interfaceCount = interfaces.length;
                    for (int j = 0; j < interfaceCount; j++) {
                        Class interfaceClass = interfaces[j];
                        if (interfaceClass.getName().equals(intializerClassName)) {
                            BdfExtensionInitializer initializer = (BdfExtensionInitializer) tmpClass.newInstance();
                            String registrationName = initializer.getRegistrationName();
                            String current = existingRegistrationNameMap.get(registrationName);
                            if (current != null) {
                                throw new InitException("same registration name: " + registrationName + " (" + current + " / " + initializer.getClass().getName() + ")");
                            } else {
                                existingRegistrationNameMap.put(registrationName, initializer.getClass().getName());
                            }
                            bdfExtensionInitializerList.add(initializer);
                        }
                    }
                }
            }
        }
        return bdfExtensionInitializerList;
    }

    public static ResourceStorages initWebappsResourceStorages(Conf conf, List<BdfExtensionInitializer> bdfExtensionInitializerList) {
        File resourceEtcDir = conf.getDir(ConfConstants.ETC_RESOURCES);
        ResourceStorage[] array = new ResourceStorage[2];
        array[0] = new DirectoryResourceStorage(ResourceStorage.ETC_NAME, resourceEtcDir, false, null);
        array[1] = DistResourceStorage.newInstance(bdfExtensionInitializerList);
        return StorageUtils.toResourceStorages(array);
    }

    private static ResourceStorages initBdfServerResourceStorages(BdfServerDirs dirs, ResourceStorages webappsResourceStorages) {
        String subdirName = "rscdata";
        File resourceVarDir = dirs.getSubPath(ConfConstants.VAR_DATA, subdirName);
        File resourceBackupDir = dirs.getSubPath(ConfConstants.VAR_BACKUP, subdirName);
        testOldVersion(resourceVarDir, resourceBackupDir);
        List<ResourceStorage> list = new ArrayList<ResourceStorage>();
        list.add(new DirectoryResourceStorage(ResourceStorage.VAR_NAME, resourceVarDir, true, resourceBackupDir));
        list.addAll(webappsResourceStorages);
        return StorageUtils.toResourceStorages(list.toArray(new ResourceStorage[list.size()]));
    }


    /*
     * Avant la révision 798 : les fichiers logos étaient directement placés dans le répertoire => on les déplace vers le répertoire images
     * Avant la révision 1198 : les répertoires commençaient par un trait de soulignement (exemple : _rsc : on l'enlève)
     * Avant la révision 3093 : on renomme rsc en images
     */
    private static void testOldVersion(File varRscDirectory, File backupRscDirectory) {
        if (varRscDirectory.exists()) {
            File[] fileArray = varRscDirectory.listFiles();
            int length = fileArray.length;
            for (int i = 0; i < length; i++) { //r1198
                File f = fileArray[i];
                if (f.isDirectory()) {
                    String name = f.getName();
                    if ((name.startsWith("_")) && (name.length() > 1)) {
                        f.renameTo(new File(varRscDirectory, name.substring(1)));
                    }

                }
            }
            File rscDir = new File(varRscDirectory, "rsc");
            File imagesDir = new File(varRscDirectory, "images"); // r3093
            if (rscDir.exists()) {
                rscDir.renameTo(imagesDir);
            }
            for (int i = 0; i < length; i++) { //r798
                File f = fileArray[i];
                if (!f.isDirectory()) {
                    String name = f.getName();
                    if ((name.endsWith(".png")) || (name.endsWith(".jpg"))) {
                        if (!imagesDir.exists()) {
                            imagesDir.mkdir();
                        }
                        f.renameTo(new File(imagesDir, name));
                    }
                }
            }
        }
        if (backupRscDirectory.exists()) {
            File[] fileArray = backupRscDirectory.listFiles();
            int length = fileArray.length;
            for (int i = 0; i < length; i++) { //r1198
                File f = fileArray[i];
                if (f.isDirectory()) {
                    String name = f.getName();
                    if ((name.startsWith("_")) && (name.length() > 1)) {
                        f.renameTo(new File(backupRscDirectory, name.substring(1)));
                    }

                }
            }
            File rscDir = new File(backupRscDirectory, "rsc");
            File imagesDir = new File(backupRscDirectory, "images"); // r3093
            if (rscDir.exists()) {
                rscDir.renameTo(imagesDir);
            }
        }
    }

    /*
     * Avant la révision 3888, les balayages étaient stockés dans etc, cette fonction les transfère si balayagedata est vide
     */
    private static void checkBalayage(BdfServerDirs dirs) {
        File varBalayageDir = dirs.getSubPath(ConfConstants.VAR_DATA, "balayagedata");
        if (!varBalayageDir.exists()) {
            String oldBalayagesPath;
            if (dirs.getName().equals(ConfConstants.UNIQUE_NAME)) {
                oldBalayagesPath = "balayages";
            } else {
                oldBalayagesPath = dirs.getName() + "/balayages";
            }
            File etcBalayageDir = dirs.getSubPath(ConfConstants.ETC_ROOT, oldBalayagesPath);
            if (etcBalayageDir.exists()) {
                try {
                    FileUtils.copyDirectory(etcBalayageDir, varBalayageDir, true);
                } catch (IOException ioe) {
                    throw new BdfStorageException(ioe);
                }
            }
        }

    }


}
