/* BDF - Copyright (c) 2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstruction;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.session.LoginException;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.multi.MultiConf;
import fr.exemole.bdfserver.multi.MultiResponseHandlerFactory;
import fr.exemole.bdfserver.multi.api.Multi;
import fr.exemole.bdfserver.multi.api.MultiConstants;
import fr.exemole.bdfserver.multi.tools.CrossInfo;
import fr.exemole.bdfserver.tools.apps.AppConf;
import fr.exemole.bdfserver.tools.apps.AppConfUtils;
import java.io.IOException;
import java.text.ParseException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.fichotheque.SubsetKey;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.CheckedNameSpace;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.localisation.LangPreferenceBuilder;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.OutputInfo;
import net.mapeadores.util.request.RequestUtils;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.HttpServletRequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public final class BDFRoutes {

    private final static CheckedNameSpace INDEX_NAMESPACE = CheckedNameSpace.build("index");
    private final static AttributeKey SPHERES_KEY = AttributeKey.build(INDEX_NAMESPACE, "spheres");

    private BDFRoutes() {

    }

    public static void resolveMulti(BDFWebapps bdfWebapps, String pathInfo, HttpServletRequestMap requestMap, HttpServletResponse response) throws IOException, ServletException {
        Multi multi = bdfWebapps.getMulti();
        int idx = pathInfo.indexOf('/');
        if (idx == -1) {
            resolveMultiRoute(bdfWebapps, checkAlias(pathInfo), requestMap, response);
        } else {
            String pathRoot = pathInfo.substring(0, idx);
            String relativePath = pathInfo.substring(idx + 1);
            switch (pathRoot) {
                case "multi-rsc":
                case "_rsc": {
                    ResponseHandler responseHandler = MultiResponseHandlerFactory.getResourceResponseHandler(multi, relativePath);
                    if (responseHandler != null) {
                        responseHandler.handleResponse(response);
                    } else {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    break;
                }
                default: {
                    BdfServer bdfServer = multi.getBdfServer(pathRoot);
                    if (bdfServer == null) {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    if (relativePath.isEmpty()) {
                        BDFUtils.sendRedirect(response, Domains.SESSION);
                    } else {
                        BDFRoutes.resolveBdfServer(bdfWebapps, bdfServer, relativePath, requestMap, response);
                    }
                }
            }
        }
    }

    private static void resolveMultiRoute(BDFWebapps bdfWebapps, String route, HttpServletRequestMap requestMap, HttpServletResponse response) throws IOException, ServletException {
        Multi multi = bdfWebapps.getMulti();
        int idx2 = route.indexOf('-');
        if (idx2 == -1) {
            if (multi.containsFichotheque(route)) {
                BDFUtils.sendRedirect(response, route + "/" + Domains.SESSION);
            } else {
                throw new HttpAccessException(HttpAccessException.NOT_FOUND);
            }
        } else {
            String firstPart = route.substring(0, idx2);
            String secondPart = route.substring(idx2 + 1);
            ResponseHandler responseHandler;
            switch (firstPart) {
                case "multi": {
                    if (secondPart.equals("admin")) {
                        responseHandler = resolveAdminMulti(bdfWebapps, requestMap);
                    } else {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    break;
                }
                case "app": {
                    AppConf appConf = AppConfUtils.getAppConf(multi.getWebappsResourceStorages(), secondPart);
                    if ((appConf != null) && (appConf.getBoolean(AppConf.MULTI_ACTIVE))) {
                        String crossLogin = appConf.getString(AppConf.MULTI_CROSSLOGIN);
                        if ((crossLogin != null) && (multi.isAuthentificationSharingAllowed())) {
                            SubsetKey sphereKey = getValidSphere(multi, crossLogin);
                            if (sphereKey != null) {
                                CrossInfo crossInfo = CrossInfoFactory.newInstance(bdfWebapps, sphereKey, requestMap);
                                if (crossInfo.hasBdfUser()) {
                                    responseHandler = MultiResponseHandlerFactory.getAppHtmlRespondeHandler(multi, appConf, crossInfo.getBdfUser());
                                } else {
                                    responseHandler = MultiResponseHandlerFactory.getCrossLoginHtmlResponseHandler(multi, crossInfo, requestMap);
                                }
                            } else {
                                throw new HttpAccessException(HttpAccessException.INTERNAL_ERROR, "wrong value for multi_crosslogin parameter");
                            }
                        } else {
                            responseHandler = MultiResponseHandlerFactory.getAppHtmlRespondeHandler(multi, appConf, null);
                        }
                    } else {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    break;
                }
                case "index": {
                    if (!multi.isAuthentificationSharingAllowed()) {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    SubsetKey sphereKey = getValidSphere(multi, secondPart);
                    if (sphereKey != null) {
                        responseHandler = resolveSphereIndex(bdfWebapps, sphereKey, requestMap);
                    } else {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    break;
                }
                default:
                    throw new HttpAccessException(HttpAccessException.NOT_FOUND);
            }
            responseHandler.handleResponse(response);
        }
    }

    public static void resolveBdfServer(BDFWebapps bdfWebapps, BdfServer bdfServer, String pathInfo, HttpServletRequestMap requestMap, HttpServletResponse response) throws IOException, ServletException {
        HttpServletRequest request = requestMap.getRequest();
        if (bdfServer.getInitState() == BdfServer.NOADMIN_STATE) {
            HttpAccessException exception = new HttpAccessException(HttpAccessException.INTERNAL_ERROR, "No Admin State", LogUtils.error("_ error.unsupported.noadminstate"));
            BDFUtils.sendError(request, response, exception, requestMap, bdfServer, null);
            return;
        }
        BdfInstruction bdfInstruction = BDFUtils.getBdfInstruction(bdfServer, requestMap, pathInfo);
        if (bdfInstruction == null) {
            HttpAccessException exception = new HttpAccessException(HttpAccessException.NOT_FOUND);
            BDFUtils.sendError(request, response, exception, requestMap, bdfServer, null);
            return;
        }
        BdfUser bdfUser = null;
        short needBdfUser = bdfInstruction.getBdfUserNeed();
        if (needBdfUser != BdfInstructionConstants.NONE_BDFUSER) {
            try {
                bdfUser = BDFUtils.getCurrentBdfUser(bdfServer, request);
                if (needBdfUser == BdfInstructionConstants.EXIT_BDFUSER) {
                    if (bdfUser != null) {
                        boolean all = false;
                        if ((bdfWebapps.isMultiBdf()) && (bdfWebapps.getMulti().isAuthentificationSharingAllowed())) {
                            all = true;
                        }
                        bdfWebapps.invalidateBdfUser(bdfServer, bdfUser, all);
                    }
                    throw new LoginException(LoginException.EXIT_ERROR);
                } else if (bdfUser == null) {
                    bdfUser = bdfWebapps.getMatchingBdfUser(bdfServer, requestMap);
                    if (bdfUser == null) {
                        bdfUser = bdfWebapps.createBdfUser(bdfServer, requestMap);
                    }
                }
            } catch (LoginException le) {
                if (needBdfUser != BdfInstructionConstants.IF_ANY_BDFUSER) {
                    ResponseHandler responseHandler = LoginEngine.resolveLoginException(bdfServer, requestMap, pathInfo, bdfInstruction, le);
                    responseHandler.handleResponse(response);
                    return;
                }
            }
        }
        try {
            ResponseHandler responseHandler = bdfInstruction.runInstruction(bdfUser);
            if (responseHandler == null) {
                throw new HttpAccessException(HttpAccessException.NOT_FOUND);
            }
            long lastModified = responseHandler.getLastModified();
            if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
                response.setHeader("Cache-Control", "no-store");
            } else if (lastModified > 0) {
                response.setDateHeader("Last-Modified", lastModified);
                try {
                    long since = request.getDateHeader("If-Modified-Since");
                    if (since >= lastModified) {
                        response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                        return;
                    }
                } catch (IllegalArgumentException iae) {
                }
            }
            responseHandler.handleResponse(response);
        } catch (HttpAccessException hae) {
            BDFUtils.sendError(request, response, hae, requestMap, bdfServer, bdfUser);
        }
    }

    private static ResponseHandler resolveAdminMulti(BDFWebapps bdfWebapps, HttpServletRequestMap requestMap) throws IOException, ServletException {
        OutputInfo outputInfo = OutputInfo.buildFromRequest(requestMap, "admin");
        Multi multi = bdfWebapps.getMulti();
        MultiConf multiConf = multi.getMultiConf();
        if (outputInfo.getType() == OutputInfo.PAGE_TYPE) {
            multiConf.update();
            if (requestMap.isTrue(MultiConstants.RELOAD_PARAMNAME)) {
                multi.getWebappsJsAnalyser().clearCache();
            }
        }
        ResponseHandler errorResponseHandler = checkInitErrorHandler(multi, outputInfo, requestMap);
        if (errorResponseHandler != null) {
            return errorResponseHandler;
        }
        switch (outputInfo.getType()) {
            case OutputInfo.JSON_TYPE:
                short check = bdfWebapps.checkMultiAdmin(requestMap);
                return MultiResponseHandlerFactory.getAdminJsonRespondeHandler(multi, check, requestMap, outputInfo.getOutput());
            case OutputInfo.STREAM_TYPE:
                return MultiResponseHandlerFactory.getAdminStreamRespondeHandler(multi, outputInfo.getOutput());
            default:
                return MultiResponseHandlerFactory.getAdminHtmlRespondeHandler(multi);
        }
    }

    private static ResponseHandler resolveSphereIndex(BDFWebapps bdfWebapps, SubsetKey sphereKey, HttpServletRequestMap requestMap) throws IOException, ServletException {
        OutputInfo outputInfo = OutputInfo.buildFromRequest(requestMap, sphereKey.getSubsetName());
        Multi multi = bdfWebapps.getMulti();
        ResponseHandler errorResponseHandler = checkInitErrorHandler(multi, outputInfo, requestMap);
        if (errorResponseHandler != null) {
            return errorResponseHandler;
        }
        CrossInfo crossInfo = CrossInfoFactory.newInstance(bdfWebapps, sphereKey, requestMap);
        String cmd = requestMap.getParameter("cmd");
        if ((cmd != null) && (cmd.equals("exit"))) {
            if (crossInfo.hasBdfUser()) {
                BdfServer bdfServer = multi.getBdfServer(crossInfo.getReferenceFichothequeName());
                bdfWebapps.invalidateBdfUser(bdfServer, crossInfo.getBdfUser(), true);
                crossInfo = crossInfo.cloneWithoutBdfUser();
            }
        }
        if (crossInfo.hasBdfUser()) {
            switch (outputInfo.getType()) {
                case OutputInfo.JSON_TYPE:
                    return MultiResponseHandlerFactory.getUserJsonRespondeHandler(multi, crossInfo.getBdfUser(), requestMap, outputInfo.getOutput());
                default:
                    return MultiResponseHandlerFactory.getIndexHtmlResponseHandler(multi, crossInfo);
            }
        } else {
            switch (outputInfo.getType()) {
                case OutputInfo.JSON_TYPE:
                    return MultiResponseHandlerFactory.getCrossLoginJsonResponseHandler(multi, crossInfo, requestMap, outputInfo.getOutput());
                default: {
                    return MultiResponseHandlerFactory.getCrossLoginHtmlResponseHandler(multi, crossInfo, requestMap);
                }
            }
        }
    }

    private static SubsetKey getValidSphere(Multi multi, String secondPart) {
        try {
            SubsetKey sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, secondPart);
            Attribute attribute = multi.getMultiMetadata().getAttributes().getAttribute(SPHERES_KEY);
            if (attribute != null) {
                if (attribute.contains(sphereKey.getSubsetName())) {
                    return sphereKey;
                }
            }
            return null;
        } catch (ParseException pe) {
            return null;
        }

    }

    private static ResponseHandler checkInitErrorHandler(Multi multi, OutputInfo outputInfo, HttpServletRequestMap requestMap) {
        CommandMessage initErrorMessage = multi.getMultiConf().getInitErrorMessage();
        if (initErrorMessage != null) {
            if (outputInfo.getType() == OutputInfo.JSON_TYPE) {
                return CommandMessageResponseHandler.getJsonInstance(multi.getAdminMessageLocalisation(), requestMap, initErrorMessage);
            } else if (initErrorMessage.getMessageKey().equals(MultiConf.MISSING_FILE_ERROR)) {
                return MultiResponseHandlerFactory.getIniResponseHandler(multi, requestMap);
            } else {
                return CommandMessageResponseHandler.getHtmlInstance(multi.getAdminMessageLocalisation(), initErrorMessage);
            }
        } else {
            return null;
        }
    }

    public static LangPreference getLangPreference(Multi multi, HttpServletRequestMap requestMap) {
        LangPreferenceBuilder langPreferenceBuilder = new LangPreferenceBuilder();
        RequestUtils.checkLangPreference(requestMap, langPreferenceBuilder);
        langPreferenceBuilder.addLangs(multi.getMultiMetadata().getWorkingLangs());
        langPreferenceBuilder.addLang(Lang.build("fr"));
        return langPreferenceBuilder.toLangPreference();
    }

    private static String checkAlias(String route) {
        switch (route) {
            case "_admin":
            case "admin-multi":
                return "multi-admin";
            default:
                return route;
        }
    }

}
