/* BdfServer - Copyright (c) 2014-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.SpecialIncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import java.io.IOException;
import java.util.List;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.mapeadores.util.xml.XsltWriter;


/**
 *
 * @author Vincent Calame
 */
public class SimpleFicheXsltWriter extends XsltWriter {

    public SimpleFicheXsltWriter() {
    }

    public void add(BdfServer bdfServer, Corpus corpus, boolean withCorpusCondition, boolean withIf) throws IOException {
        if (corpus == null) {
            openMatchXslTemplate("fiche", null);
            closeXslTemplate();
            return;
        }
        if (withCorpusCondition) {
            openMatchXslTemplate("fiche[@corpus='" + corpus.getSubsetName() + "']", null);
        } else {
            openMatchXslTemplate("fiche", null);
        }
        UiManager uiManager = bdfServer.getUiManager();
        UiComponents uiComponents = uiManager.getMainUiComponents(corpus);
        boolean withSoustitre = uiComponents.contains(FieldKey.SOUSTITRE);
        List<Corpus> includedCorpusList = BdfServerUtils.getIncludeSatelliteList(corpus);
        List<UiComponent> uiComponentList = uiComponents.getUiComponentList();
        if (!includedCorpusList.isEmpty()) {
            uiComponentList = UiUtils.filter(uiComponentList, UiUtils.MAIN_COMPONENTPREDICATE);
        }
        boolean uniqueLang = BdfTransformationUtils.isUniquelang(bdfServer);
        appendUiComponents(corpus, uiComponentList, withIf, uniqueLang, withSoustitre, includedCorpusList);
        closeXslTemplate();
        for (Corpus satellite : includedCorpusList) {
            UiComponents satelliteUiComponents = uiManager.getMainUiComponents(satellite);
            List<UiComponent> satelliteList = UiUtils.filter(satelliteUiComponents.getUiComponentList(), UiUtils.SATELLITE_COMPONENTPREDICATE);
            openMatchXslTemplate("fiche[@corpus='" + satellite.getSubsetName() + "']", "_satellite");
            addSatelliteUiComponents(satellite, satelliteList, uniqueLang, withSoustitre);
            closeXslTemplate();
        }
    }

    private void addSatelliteUiComponents(Corpus satellite, List<UiComponent> satelliteList, boolean withIf, boolean uniqueLang) throws IOException {
        CorpusMetadata corpusMetadata = satellite.getCorpusMetadata();
        String subsetName = satellite.getSubsetName();
        addXslVariable("label", "$LABELS/satellite[@corpus='" + subsetName + "']/label", false);
        addXslVariable("hide", "$USER/ui[@corpus='" + subsetName + "']/hide", false);
        startOpenTag("section");
        addAttribute("class", "fiche-satellite-Block");
        endOpenTag();
        startOpenTag("h1");
        addAttribute("class", "fiche-satellite-Title");
        endOpenTag();
        addXslValueOf("$LABELS/satellite[@corpus='" + subsetName + "']/@title");
        addXslApplyTemplates("soustitre", "MODE_INLINE");
        closeTag("h1", false);
        for (UiComponent uiComponent : satelliteList) {
            if (uiComponent instanceof FieldUi) {
                appendFieldUi((FieldUi) uiComponent, withIf, uniqueLang, corpusMetadata);
            } else if (uiComponent instanceof SpecialIncludeUi) {
                appendSpecialIncludeUi((SpecialIncludeUi) uiComponent, withIf);
            } else if (uiComponent instanceof SubsetIncludeUi) {
                appendSubsetIncludeUi((SubsetIncludeUi) uiComponent, withIf);
            } else if (uiComponent instanceof CommentUi) {
                CommentUi commentUi = (CommentUi) uiComponent;
                if ((commentUi.getLocation() & CommentUi.TEMPLATE_BITVALUE) != 0) {
                    addXslValueOf("$label[@name='comment_" + commentUi.getCommentName() + "']", true, true);
                }
            }
        }
        closeTag("section");
    }

    private void appendUiComponents(Corpus corpus, List<UiComponent> uiComponentList, boolean withIf, boolean uniqueLang, boolean withSoustitre, List<Corpus> includedCorpusList) throws IOException {
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        String subsetName = corpus.getSubsetName();
        addXslVariable("label", "$LABELS[@corpus='" + subsetName + "']/label", false);
        addXslVariable("hide", "$USER/ui[@corpus='" + subsetName + "']/hide", false);
        openTag("article");
        startOpenTag("header");
        addAttribute("class", "fiche-Header");
        endOpenTag();
        openTag("h1");
        addXslApplyTemplates(".", "MODE_TITLE");
        closeTag("h1", false);
        if (withSoustitre) {
            if (withIf) {
                openXslIf("$SHOWEMPTY or soustitre", true);
                appendSoustitre();
                closeXslIf(true);
            } else {
                appendSoustitre();
            }
        }
        closeTag("header");
        for (UiComponent uiComponent : uiComponentList) {
            if (uiComponent instanceof FieldUi) {
                appendFieldUi((FieldUi) uiComponent, withIf, uniqueLang, corpusMetadata);
            } else if (uiComponent instanceof SpecialIncludeUi) {
                appendSpecialIncludeUi((SpecialIncludeUi) uiComponent, withIf);
            } else if (uiComponent instanceof SubsetIncludeUi) {
                appendSubsetIncludeUi((SubsetIncludeUi) uiComponent, withIf);
            } else if (uiComponent instanceof CommentUi) {
                CommentUi commentUi = (CommentUi) uiComponent;
                if ((commentUi.getLocation() & CommentUi.TEMPLATE_BITVALUE) != 0) {
                    addXslValueOf("$label[@name='comment_" + commentUi.getCommentName() + "']", true, true);
                }
            }
        }
        for (Corpus satellite : includedCorpusList) {
            addXslApplyTemplates("fiche[@corpus='" + satellite.getSubsetName() + "']", "_satellite");
        }
        appendChrono();
        closeTag("article");
    }

    private void appendSoustitre() throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Description");
        endOpenTag();
        addXslApplyTemplates("soustitre", "MODE_INLINE");
        closeTag("p", false);
    }

    private void appendSpecialIncludeUi(SpecialIncludeUi includeUi, boolean withIf) throws IOException {
        LimitOption limitOption = includeUi.getLimitOption();
        if ((limitOption != null) && (limitOption.isNoNewAllowed())) {
            withIf = true;
        }
        switch (includeUi.getName()) {
            case FichothequeConstants.LIAGE_NAME:
                if (withIf) {
                    openXslIf("not($hide[@name='liage']) and ($SHOWEMPTY or extraitcorpus[@extrait-name='liage'])", true);
                    appendLiage();
                    closeXslIf(true);
                } else {
                    appendLiage();
                }
                break;
            case FichothequeConstants.RATTACHEMENT_NAME:
                appendRattachement();
                break;
        }
    }

    private void appendSubsetIncludeUi(SubsetIncludeUi includeUi, boolean withIf) throws IOException {
        LimitOption limitOption = includeUi.getLimitOption();
        if ((limitOption != null) && (limitOption.isNoNewAllowed())) {
            withIf = true;
        }
        short category = includeUi.getExtendedIncludeKey().getCategory();
        if (withIf) {
            openXslIf("not($hide[@name='" + includeUi.getName() + "']) and ($SHOWEMPTY or extrait" + SubsetKey.categoryToString(category) + "[@extrait-name='" + includeUi.getName() + "'])", true);
        }
        switch (category) {
            case SubsetKey.CATEGORY_THESAURUS:
                if (includeUi.getInputOption().getInputType().equals(InputOption.INPUT_FICHESTYLE)) {
                    appendListSubset(includeUi);
                } else {
                    appendInlineSubset(includeUi);
                }
                break;
            case SubsetKey.CATEGORY_ALBUM:
                appendInlineSubset(includeUi);
                break;
            default:
                appendListSubset(includeUi);
        }
        if (withIf) {
            closeXslIf(true);
        }
    }


    private void appendInlineSubset(SubsetIncludeUi includeUi) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Field");
        endOpenTag();
        startOpenTag("span", false);
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='" + includeUi.getName() + "']");
        append(" : ");
        closeTag("span", false);
        addXslApplyTemplates("extrait" + SubsetKey.categoryToString(includeUi.getCategory()) + "[@extrait-name='" + includeUi.getName() + "']", "MODE_INLINE");
        closeTag("p", false);
    }

    private void appendListSubset(SubsetIncludeUi includeUi) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='" + includeUi.getName() + "']");
        append(" :");
        closeTag("p", false);
        startOpenTag("ul");
        addAttribute("class", "fiche-Block");
        endOpenTag();
        addXslApplyTemplates("extrait" + SubsetKey.categoryToString(includeUi.getCategory()) + "[@extrait-name='" + includeUi.getName() + "']", "MODE_LISTITEM", true);
        closeTag("ul");
    }

    private void appendLiage() throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='liage']");
        append(" :");
        closeTag("p", false);
        startOpenTag("ul");
        addAttribute("class", "fiche-Block");
        endOpenTag();
        addXslApplyTemplates("extraitcorpus[@extrait-name='liage']", "MODE_LISTITEM", true);
        closeTag("ul");
    }

    private void appendRattachement() throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='rattachement']");
        append(" :");
        closeTag("p", false);
        startOpenTag("ul");
        addAttribute("class", "fiche-Block");
        endOpenTag();
        addXslApplyTemplates("motcle", "MODE_LISTITEM_TEXTONLY", true);
        addXslApplyTemplates("fiche", "MODE_LISTITEM", true);
        closeTag("ul");
    }

    private void appendFieldUi(FieldUi fieldUi, boolean withIf, boolean uniqueLang, CorpusMetadata corpusMetadata) throws IOException {
        FieldKey fieldKey = fieldUi.getFieldKey();
        switch (fieldKey.getCategory()) {
            case FieldKey.PROPRIETE_CATEGORY: {
                if (withIf) {
                    openXslIf("$SHOWEMPTY or propriete[@name='" + fieldKey.getFieldName() + "']", true);
                    appendPropriete(fieldKey);
                    closeXslIf(true);
                } else {
                    appendPropriete(fieldKey);
                }
                break;
            }
            case FieldKey.INFORMATION_CATEGORY: {
                CorpusField corpusField = corpusMetadata.getCorpusField(fieldKey);
                if (withIf) {
                    openXslIf("$SHOWEMPTY or information[@name='" + fieldKey.getFieldName() + "']", true);
                    appendInformation(fieldKey, corpusField);
                    closeXslIf(true);
                } else {
                    appendInformation(fieldKey, corpusField);
                }
                break;
            }
            case FieldKey.SECTION_CATEGORY: {
                if (withIf) {
                    openXslIf("$SHOWEMPTY or section[@name='" + fieldKey.getFieldName() + "']", true);
                    appendSection(fieldKey);
                    closeXslIf(true);
                } else {
                    appendSection(fieldKey);
                }
                break;
            }
            case FieldKey.SPECIAL_CATEGORY: {
                switch (fieldKey.getFieldString()) {
                    case FieldKey.SPECIAL_LANG: {
                        if (!uniqueLang) {
                            appendLang();
                        }
                        break;
                    }
                    case FieldKey.SPECIAL_REDACTEURS: {
                        if (withIf) {
                            openXslIf("$SHOWEMPTY or redacteurs", true);
                            appendRedacteurs();
                            closeXslIf(true);
                        } else {
                            appendRedacteurs();
                        }
                        break;
                    }
                }
                break;
            }
        }
    }

    private void appendLang() throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Field");
        endOpenTag();
        startOpenTag("span", false);
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='lang']");
        append(" : ");
        closeTag("span", false);
        addXslApplyTemplates("lang/langue", null);
        closeTag("p", false);
    }

    private void appendSection(FieldKey fieldKey) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='section_" + fieldKey.getFieldName() + "']");
        append(" :");
        closeTag("p", false);
        startOpenTag("section");
        addAttribute("class", "fiche-Section");
        endOpenTag();
        addXslApplyTemplates("section[@name='" + fieldKey.getFieldName() + "']", null, true);
        closeTag("section");
    }

    private void appendPropriete(FieldKey fieldKey) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Field");
        endOpenTag();
        startOpenTag("span", false);
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='propriete_" + fieldKey.getFieldName() + "']");
        append(" : ");
        closeTag("span", false);
        addXslApplyTemplates("propriete[@name='" + fieldKey.getFieldName() + "']", "MODE_INLINE");
        closeTag("p", false);
    }

    private void appendInformation(FieldKey fieldKey, CorpusField corpusField) throws IOException {
        if ((corpusField != null) && (CorpusMetadataUtils.isBlockDisplayInformationField(corpusField))) {
            switch (corpusField.getFicheItemType()) {
                case CorpusField.ITEM_FIELD:
                case CorpusField.PARA_FIELD:
                case CorpusField.IMAGE_FIELD:
                    appendBlockInformation(fieldKey);
                    break;
                default:
                    appendListInformation(fieldKey);
            }
        } else {
            appendInlineInformation(fieldKey);
        }
    }

    private void appendInlineInformation(FieldKey fieldKey) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Field");
        endOpenTag();
        startOpenTag("span", false);
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='information_" + fieldKey.getFieldName() + "']");
        append(" : ");
        closeTag("span", false);
        addXslApplyTemplates("information[@name='" + fieldKey.getFieldName() + "']", "MODE_INLINE");
        closeTag("p", false);
    }

    private void appendBlockInformation(FieldKey fieldKey) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='information_" + fieldKey.getFieldName() + "']");
        append(" :");
        closeTag("p", false);
        startOpenTag("div");
        addAttribute("class", "fiche-Block");
        endOpenTag();
        addXslApplyTemplates("information[@name='" + fieldKey.getFieldName() + "']", "MODE_BLOCK", true);
        closeTag("div");
    }

    private void appendListInformation(FieldKey fieldKey) throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='information_" + fieldKey.getFieldName() + "']");
        append(" :");
        closeTag("p", false);
        startOpenTag("ul");
        addAttribute("class", "fiche-Block");
        endOpenTag();
        addXslApplyTemplates("information[@name='" + fieldKey.getFieldName() + "']", "MODE_LISTITEM", true);
        closeTag("ul");
    }

    private void appendRedacteurs() throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Field");
        endOpenTag();
        startOpenTag("span", false);
        addAttribute("class", "fiche-Label");
        endOpenTag();
        addXslValueOf("$label[@name='redacteurs']");
        append(" : ");
        closeTag("span", false);
        addXslApplyTemplates("redacteurs", "MODE_INLINE");
        closeTag("p", false);
    }

    private void appendChrono() throws IOException {
        startOpenTag("p");
        addAttribute("class", "fiche-Time");
        endOpenTag();
        addXslValueOf("$label[@name='date_creation']");
        append(" : ");
        addXslApplyTemplates("chrono/datation[1]", "MODE_INLINE");
        append(" — ");
        addXslValueOf("$label[@name='date_modification']");
        append(" : ");
        addXslApplyTemplates("chrono/datation[position() = last()]", "MODE_INLINE");
        closeTag("p", false);
    }

}
