/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import java.util.List;
import net.fichotheque.ExistingIdException;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.NoMasterIdException;
import net.fichotheque.tools.duplication.DuplicationEngine;
import net.fichotheque.tools.duplication.DuplicationParameters;
import net.fichotheque.tools.duplication.SubsetMatch;
import net.fichotheque.utils.CorpusUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class DuplicationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "Duplication";
    public final static String COMMANDKEY = "_ CRP-23";
    public final static String MODE_PARAMNAME = "mode";
    public final static String SELECTION_MODE_PARAMVALUE = "selection";
    public final static String LIST_MODE_PARAMVALUE = "list";
    public final static String IDTOKENS_PARAMNAME = "idtokens";
    public final static String LINKTOORIGIN_PARAMNAME = "linktoorigin";
    public final static String MODE_LINKTOORIGIN_PARAMNAME = "linktoorigin_mode";
    public final static String POIDS_LINKTOORIGIN_PARAMNAME = "linktoorigin_poids";
    public final static String DESTINATION_PARAMNAME = "destination";
    public final static String WITHDESTINATIONIDS_PARAMNAME = "withdestinationids";
    private DuplicationParameters duplicationParameters;

    public DuplicationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        SubsetMatch subsetMatch = DuplicationEngine.run(duplicationParameters, editSession.getFichothequeEditor());
        putResultObject(SUBSETMATCH_OBJ, subsetMatch);
        setDone("_ done.corpus.duplication", duplicationParameters.getOriginCorpus().getSubsetKey(), duplicationParameters.getDestinationCorpus().getSubsetKey(), subsetMatch.size());
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkFichothequeAdmin();
        Corpus corpus = getMandatoryCorpus();
        String mode = getMandatory(MODE_PARAMNAME);
        Corpus destinationCorpus = (Corpus) BdfInstructionUtils.getMandatorySubset(fichotheque, requestMap, SubsetKey.CATEGORY_CORPUS, DESTINATION_PARAMNAME);
        boolean withDestinationIds = false;
        if (mode.equals(LIST_MODE_PARAMVALUE)) {
            withDestinationIds = requestMap.isTrue(WITHDESTINATIONIDS_PARAMNAME);
        }
        if ((destinationCorpus.getMasterSubset() != null) && (!withDestinationIds)) {
            setError("_ error.unsupported.destinationidsrequired");
            return false;
        }
        this.duplicationParameters = new DuplicationParameters(corpus, destinationCorpus);
        if (mode.equals(SELECTION_MODE_PARAMVALUE)) {
            List<FicheMeta> ficheMetaList = CorpusUtils.getFicheMetaListByCorpus(bdfUser.getSelectedFiches(), corpus);
            if (ficheMetaList.isEmpty()) {
                setError("_ error.empty.originselection");
                return false;
            }
            for (FicheMeta ficheMeta : ficheMetaList) {
                duplicationParameters.addFicheMeta(ficheMeta);
            }
        } else if (mode.equals(LIST_MODE_PARAMVALUE)) {
            String idTokens = getMandatory(IDTOKENS_PARAMNAME);
            if (withDestinationIds) {
                String[] lines = StringUtils.getLineTokens(idTokens, StringUtils.EMPTY_EXCLUDE);
                if (lines.length == 0) {
                    setError("_ error.empty.idtokens");
                    return false;
                } else {
                    parseMatchLines(duplicationParameters, lines);
                }
            } else {
                String[] tokens = StringUtils.getTechnicalTokens(idTokens, true);
                if (tokens.length == 0) {
                    setError("_ error.empty.idtokens");
                    return false;
                } else {
                    parseSimpleIdTokens(duplicationParameters, tokens);
                }
            }
            if (hasMultiError()) {
                setMultiErrorTitle("_ error.list.idtokens");
                return false;
            }
        }
        boolean linkToOrigin = requestMap.isTrue(LINKTOORIGIN_PARAMNAME);
        if (linkToOrigin) {
            String linkToOriginMode = getMandatory(MODE_LINKTOORIGIN_PARAMNAME).trim();
            String linkToOriginPoids = getMandatory(POIDS_LINKTOORIGIN_PARAMNAME).trim();
            int poids = 1;
            if (linkToOriginPoids.length() > 0) {
                boolean done = false;
                try {
                    poids = Integer.parseInt(linkToOriginPoids);
                    if (poids > 0) {
                        done = true;
                    }
                } catch (NumberFormatException nfe) {
                }
                if (!done) {
                    setError("_ error.wrong.poids", linkToOriginPoids);
                    return false;
                }
            }
            try {
                duplicationParameters.setLinkWithOrigin(linkToOriginMode, poids);
            } catch (ParseException pe) {
                setError("_ error.wrong.mode", linkToOriginPoids);
                return false;
            }
        }
        return true;
    }

    private void parseSimpleIdTokens(DuplicationParameters duplicationParameters, String[] tokens) throws BdfInstructionException {
        for (String token : tokens) {
            try {
                int id = Integer.parseInt(token);
                if (id > 0) {
                    FicheMeta origin = duplicationParameters.getOriginCorpus().getFicheMetaById(id);
                    if (origin != null) {
                        duplicationParameters.addFicheMeta(origin);
                    } else {
                        addMultiError("_ error.unknown.id", token);
                    }
                } else {
                    addMultiError("_ error.wrong.id", token);
                }
            } catch (NumberFormatException nfe) {
                addMultiError("_ error.wrong.id", token);
            }
        }
    }

    private void parseMatchLines(DuplicationParameters duplicationParameters, String[] lines) throws BdfInstructionException {
        Subset masterSubset = duplicationParameters.getDestinationCorpus().getMasterSubset();
        for (String line : lines) {
            String[] tokens = StringUtils.getTechnicalTokens(line, true);
            if (tokens.length < 2) {
                addMultiError("_ error.wrong.id", line);
            } else {
                FicheMeta origin = null;
                String firstToken = tokens[0];
                try {
                    int id = Integer.parseInt(firstToken);
                    if (id > 0) {
                        origin = duplicationParameters.getOriginCorpus().getFicheMetaById(id);
                        if (origin == null) {
                            addMultiError("_ error.unknown.id", firstToken);
                        }
                    } else {
                        addMultiError("_ error.wrong.id", firstToken);
                    }
                } catch (NumberFormatException nfe) {
                    addMultiError("_ error.wrong.id", firstToken);
                }
                if (origin != null) {
                    String secondToken = tokens[1];
                    try {
                        int destinationId = Integer.parseInt(secondToken);
                        if (destinationId < 1) {
                            if (masterSubset != null) {
                                addMultiError("_ error.empty.mandatorydestinationid", secondToken);
                            } else {
                                duplicationParameters.addFicheMeta(origin);
                            }
                        } else {
                            try {
                                duplicationParameters.addFicheMeta(origin, destinationId);
                            } catch (ExistingIdException eie) {
                                addMultiError("_ error.existing.id", secondToken);
                            } catch (NoMasterIdException nmie) {
                                addMultiError("error.unknown.mastersubsetitem", secondToken, masterSubset.getSubsetKey().toString());
                            }
                        }
                    } catch (NumberFormatException nfe) {
                        addMultiError("_ error.wrong.destinationid", secondToken);
                    }
                }
            }
        }
    }

}
